#######################################################################
#
# NCM component for dirperm daemon
#
# Charles Loomis <charles.loomis@cern.ch>
# Version 1.0.4 : 07/12/04 11:22
#
# ** Generated file : do not edit **
#
# A translation of the edg-lcfg-dirperm component written by David
# Groep <davidg@nikhef.nl>. 
#
#######################################################################

package NCM::Component::dirperm;

use strict;
use NCM::Component;
use vars qw(@ISA $EC);
@ISA = qw(NCM::Component);
$EC=LC::Exception::Context->new->will_store_all;
use NCM::Check;

use EDG::WP4::CCM::Element;

use File::Path;
use File::Copy;
use File::Basename;

local(*DTA);


##########################################################################
sub Configure($$@) {
##########################################################################
    
    my ($self, $config) = @_;

    # Define paths for convenience. 
    my $base = "/software/components/dirperm";

    # If the list of paths exists, actually do something!
    if ($config->elementExists("$base/paths")) {

        # Get the list of path entries; should be a list.
        my @paths = $config->getElement("$base/paths")->getList();
        foreach my $element (@paths) {
            my %hash = $element->getHash();
            my $rc = process_path($self,\%hash);
        }
    }

    return 1;
}

sub process_path {

    my ($self, $href) = @_;
    my %record = %$href;

    # Pull out the values and check that they really are defined. 
    my $path = $record{path}->getValue();
    if (!defined($path)) {
        $self->error("entry with undefined path");
        return 0;
    }

    # Get the owner. 
    my $owner = $record{owner}->getValue();
    if (!defined($owner)) {
        $self->error("entry with undefined owner");
        return 0;
    }

    # Verify the format of owner (owner:group) and get the uid and gid.
    my $uid = undef;
    my $gid = undef;
    if ($owner =~ m/(\w+)(:(\w+))?/) {

        # Collect the uid and gid to use. 
        my $user = $1;
	my $group = undef;
	$group = $3 if ($3);

	# gid defaults to the one in the passwd file
        ($uid, $gid) = (getpwnam($user))[2,3];

	# If the group was specified, use that instead.
	$gid = (getgrnam($group))[2] if ($group);

        if (!defined($uid) or !defined($gid)) {
            $self->error("bad owner or group ($owner)");
            return 0;
        }
    } else {
        $self->error("owner field with bad format ($owner)");
        return 0;
    }
    
    # Permissions. 
    my $perm = oct($record{perm}->getValue());
    if (!defined($perm)) {
        $self->error("entry with undefined permissions");
        return 0;
    }

    # Type: f=file, d=directory.
    my $type = $record{type}->getValue();
    if (!defined($type)) {
        $self->error("entry with undefined type");
        return 0;
    }

    # Check for errors in the type of an already existing file.
    if ((-d $path) && ($type eq "f")) {
        $self->error("$path exists but isn't a directory");
        return 0;
    } 
    if ((-f $path) && ($type eq "d")) {
        $self->error("$path exists but isn't a file");
        return 0;
    } 

    # Make the file or directory.
    if ($type eq "f") {

        # Make the file.  
        open TMP, ">>$path";
        close TMP;
        if (! -f $path) {
            $self->error("error creating file $path");
            return 0;
        }

        chown $uid, $gid, $path;
        if ($?) {
            $self->error("can't change owner on $path");
            return 0;
        }

        chmod $perm, $path;
        if ($?) {
            $self->error("can't change permissions on $path");
            return 0;
        }

    } elsif ($type eq "d") {

        # Make the directory and any parent directories. 
        mkpath($path,0,$perm);
        if ($?) {
            $self->error("error making directory: $!");
            return 0;
        }

        chown $uid, $gid, $path;
        if ($?) {
            $self->error("can't change owner on $path");
            return 0;
        }

        chmod $perm, $path;
        if ($?) {
            $self->error("can't change permissions on $path");
            return 0;
        }

    } else {

        # Bad entry. 
        $self->error("bad file type ($type) given");
        return 0;
    }

    # Copy files into newly created directory if necessary.
    my $initdir = $record{initdir};
    if (defined($initdir)) {
        if ($type eq "d") {
            my @paths = $initdir->getList();
            foreach my $element (@paths) {
                my $srcdir = $element->getValue();
                
                # Ensure that this is a directory and exists.
                if (! -d $srcdir) {
                    $self->error("initdir path ($srcdir) doesn't exist");
                    next;
                }
                
                # Collect the ordinary files to copy.
                opendir DIR, "$srcdir";
                my @files = grep -T, map "$srcdir/$_", readdir DIR;
                closedir DIR;
                
                # Copy the files. This uses the default permissions
                # on the copy as the original dirperm object did. 
                foreach (@files) {
                    my $dstfile = "$path/" . basename($_);
                    copy($_,$dstfile);
                    $self->error("error copying file $_") if ($?);
                    
                    chown $uid, $gid, $dstfile;
                    if ($?) {
                        $self->error("can't change owner on $dstfile");
                        return 0;
                    }
                }
            }
        } else {
            
            # initdir has been specified on a file entry
            $self->error("initdir specified for non-directory entry");
            return 0;
        }
    }

    return 1;
}


1;      # Required for PERL modules
